import os
from pathlib import Path
import sysconfig
import sys
from typing import cast
from zscams.agent.src.core.backend.client import backend_client
from zscams.agent.src.support.configuration import reinitialize, CONFIG_PATH
from zscams.agent.src.support.logger import get_logger
from zscams.agent.src.support.os import create_system_user, install_service, is_freebsd
from zscams.agent.src.support.ssh import add_to_authorized_keys
from zscams.agent.src.support.cli import prompt

logger = get_logger("bootstrap")


def bootstrap():
    """Ensure the agent is bootstrapped with the backend."""

    if backend_client.is_bootstrapped():
        logger.info("Agent ID %s is already bootstrapped.", backend_client.agent_id)
        return

    username = input("Username: ")
    totp = input("One Time Password (OTP): ")

    backend_client.login(username, totp)

    customer_name = prompt(
        "Customer Name",
        "Customer Name",
        required=True,
        startswith="",
    )
    connector_name = prompt(
        "Connector Name",
        "Connector Name [Unique name for this connector]",
        required=True,
        startswith="",
    )
    equipment_type = prompt(
        "Equipment Type",
        "Equipment Type [Can be ZPA|VZEN|PSE]",
        required=True,
        startswith="",
    )
    equipment_name = (
        f"{equipment_type.lower()}-{customer_name.lower()}-{connector_name.lower()}"
    )
    enforced_id = prompt("Enforced ID", "Enforced Agent ID")
    if not os.path.exists(CONFIG_PATH):
        os.remove(CONFIG_PATH)
    reinitialize(equipment_name=equipment_name, equipment_type=equipment_type)
    cm_info = backend_client.bootstrap(equipment_name, enforced_id or None)

    sys_user = cast(str, cm_info.get("ssh_user"))

    create_system_user(sys_user)
    add_to_authorized_keys(sys_user, cm_info.get("server_ssh_pub_key"))
    install_zscams_systemd_service(sys_user)


def install_zscams_systemd_service(user_to_run_as: str):

    data = {
        "pythonpath": sysconfig.get_paths()["purelib"],
        "python_exec": sys.executable,
        "user_to_run_as": user_to_run_as,
    }
    import zscams

    BASE_DIR = Path(zscams.__file__).resolve().parent
    filename = "linux_service.j2" if is_freebsd() else "freebsd_service.j2"
    template_path = f"{BASE_DIR}/agent/configuration/{filename}.j2"
    with open(template_path) as f:
        template = f.read()

    rendered_config = template.format(**data)

    install_service("zscams" if is_freebsd() else "zscams.service", rendered_config)



import os
from zscams.agent.src.support.filesystem import resolve_path
from zscams.agent.src.support.configuration import get_config, CONFIG_PATH, ROOT_PATH
from zscams.agent.src.core.backend.client import BackendClient
from zscams.agent.src.support.logger import get_logger
from zscams.agent.src.support.os import remove_service, is_freebsd

logger = get_logger("Unbootstrap")


def unbootstrap():
    remote_configs = get_config().get("remote", {})
    backend_config = get_config().get("backend", {})
    private_key_path = resolve_path(
        remote_configs.get("client_key"), os.path.dirname(CONFIG_PATH)
    )

    cert_path = resolve_path(
        remote_configs.get("client_key"), os.path.dirname(CONFIG_PATH)
    )

    ca_chain_path = resolve_path(
        remote_configs.get("client_key"), os.path.dirname(CONFIG_PATH)
    )

    cache_path = os.path.join(
        ROOT_PATH.parent,
        backend_config.get("cache_dir"),
        BackendClient.MACHINE_INFO_FILE_NAME,
    )

    if os.path.exists(private_key_path):
        os.remove(private_key_path)
        logger.debug("Removed private key")

    if os.path.exists(cert_path):
        os.remove(cert_path)
        logger.debug("Removed certificate")

    if os.path.exists(ca_chain_path):
        os.remove(ca_chain_path)
        logger.debug("Removed CA Chain")

    if os.path.exists(cache_path):
        os.remove(cache_path)
        logger.debug("Removed Machine info")

    remove_service("zscams" if is_freebsd() else "zscams.service")
    logger.debug("Removed ZSCAMs service")
